/*                                                                
**  Copyright (C) 1996,2007,2011-2013,2019  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


#ifndef BAD_PIXEL_DEFS_H
#define BAD_PIXEL_DEFS_H 

#ifndef LIBERR_H
#include "liberr.h"
#define LIBERR_H
#endif

#ifndef DSLIB_H
#include "dslib.h"
#define DSLIB_H
#endif

#ifndef ASCDM_H
#include "ascdm.h"              /* data model */
#endif
 
#ifndef EVENT_DEFS_H
#include "event_defs.h"
#define EVENT_DEFS_H
#endif 


/*  the following defines are used for badpixel file column. The names below
 *  are hardwired based on the names that are anticipated to be received from
 *  telemetry and may need revision. 
 *
 *  BAD PIXEL FILE COLUMNS
 */
 
#define BAD_SHAPE_COL        "SHAPE"
#define BAD_TIME_COL         "TIME"
#define BAD_TIME_END_COL     "TIME_STOP"
#define BAD_CHIPX_COL        "CHIPX"
#define BAD_CHIPY_COL        "CHIPY"
#define BAD_STATUS_COL       "STATUS"


/*  the following defines are used for setting up arrays as well as for range
 *  checking. Therefore the numbers are actually  non-inclusive so as to allow
 *  direct indexing of values. Ie. max values are +1 and min values are -1 from
 *  the valid values.  
 * 
 *  RANGE DEFINES
 */
#define BP_MIN_COORD                 0  
#define BP_MAX_COORD              1025 
#define BP_MIN_CCD                   0
#define BP_MAX_CCD                   9  
#define BP_MAX_ACTIVE_CCDS           6


#define BP_CORNER_MEAN_MISSING     4095
/*#define BP_STATUS_4_MASK         ((unsigned long) 0x178FF)*/
#define BP_STATUS_4_MASK         ((unsigned long) 0x378FF)
#define BP_STATUS_5_MASK         ((unsigned long) 0x500)
#define BP_STATUS_16_MASK        ((unsigned long) 0x8000)


#define VF_BADPIX_MASK           ((unsigned long) 0x0001781F)
#define L1_STS_BP_TRUE  (L1_STS_CHAIN_2 | L1_STS_LM_BAD_PIX | L1_STS_ADJ_BAD_PIX)
#define CENTROID_BADPIX ((unsigned long)0x1E81F)
#define L1_STS_BP_VF   ((unsigned long)0x80000000)

typedef unsigned long  BADPIX_MASK_T[BP_MAX_COORD][BP_MAX_COORD]; 
typedef double BADPIX_ENDTIME_T[BP_MAX_COORD][BP_MAX_COORD]; 


/*
 *
 */
typedef struct badpix_entry_t {
   dmBlock*      ext;      /* data extension handle */
   dmDescriptor* shape;    /*  */
   dmDescriptor* x_col;    /*  */
   dmDescriptor* y_col;    /*  */
   dmDescriptor* time_col; /*  */
   dmDescriptor* time_end_col; /*  */
   dmDescriptor* sts_col;  /* status column data descriptor */
   double        time;     /* value of start time field in bad pix data */
   double        time_end; /* value of end time field in bad pix data */
   long          check;/* have all rows been read yet */
   int           rows; /* number of rows in the event file */
   short         chipx[2]; /* value of chipx in bad pix data */
   short         chipy[2]; /* value of chipy in bad pix data */
   unsigned long status;   /* value of status mask (32x) */
   boolean       exists; /* extension exists for this ccd */
   char          type[DS_SZ_KEYWORD];  /* point or rectangle */ 
} BADPIX_ENTRY_T, *BADPIX_ENTRY_P_T; 


/*  the following structure contains all of the data fields necessary to
 *  read the bad pixel file.
 *
 *  BAD PIXEL HOUSE KEEPING STUCTURE
 */

typedef struct badpix_setup_t {
   dmDataset*       dataset;         /* dataset (file) handle */
   BADPIX_ENTRY_T   ccd[NUM_CCDS];   /* ccd specific bad pixel data */ 
   short            mapping[NUM_CCDS];   /* bad pixel table for active ccds */
   short            ccd_accessed[NUM_CCDS];
   short            in_use;
   BADPIX_MASK_T    mask[BP_MAX_ACTIVE_CCDS];  /* bad pixel tables */
   BADPIX_ENDTIME_T endtime[BP_MAX_ACTIVE_CCDS];   /* bad pixel tables */
} BADPIX_SETUP_T, *BADPIX_SETUP_P_T;


/* function to open the bad pixel dataset and initialize structure */
extern void set_badpix_map_values (char*,
                                     BADPIX_SETUP_P_T,
                                     dsErrList*);

/* function to add bad pixel elements to the internal bad pixel map */
extern void update_badpix_map(EVENT_REC_P_T,
                              BADPIX_SETUP_P_T,
                              dsErrList*,
                              bool);

/* function to check if an event lands on a bad pixel */
extern void check_event_for_badpix(EVENT_REC_P_T,
                                   short, 
                                   BADPIX_SETUP_P_T,
                                   dsErrList*); 

/* function to clean up the bad pixel file */
extern void cleanup_bad_pix(BADPIX_SETUP_P_T, 
                            dsErrList*);

#endif 
