/*                                                                
**  Copyright (C) 1998,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */


#ifndef BIAS_MAP_DEFS_H
#define BIAS_MAP_DEFS_H 

#include "bool.h"

/* data model */
#ifndef ASCDM_H
#include "ascdm.h"              /* data model */
#endif

#define CCD_INDEXES      11  /* max number of CDD's + 1 for direct indexing */
#define NO_BIAS_MAPS   0
#define USE_BIAS_MAPS  1

/*  The following structure is used by acis level 1 tools to store information
 *  pertaining to bias image files.  
 *
 *  BIAS_MAP FILE SETUP STRUCTURE
 */
typedef struct bias_setup_t {
   dmBlock*      primary;   /* header keyword extension handle */
   dmBlock*      image;     /* image data extension handle */
   dmDescriptor*  data;    /* event attributes */
   long          axlen[2];  /* dimension of image axes           */
   long          img_LL[2]; /* lower left corner of image */
   long          img_UR[2]; /* upper right corner of image */
   long          img_area;  /* image size (axis1 * axis2) in elements */ 
   boolean     open_map;        /* True if bias map has been loaded */
   short       bias_state;      /* use this bias map or not */
   short       ccdid;           /* ccd id of bias map */
   short       occurances;      /* number of occurances of this map in stack */
   short         init_oc[4]; /* initial overclock values A-D */ 
   dmDataType*   types;     /* ptr to array of event data column types */
   short*        biasmap;   /* bias map image */
   char*         file;      /* name of bias map file */ 
} BIAS_SETUP_T, *BIAS_SETUP_P_T;
 
typedef BIAS_SETUP_T BIAS_SETUP_A_T[CCD_INDEXES];


extern unsigned short load_bias_image(BIAS_SETUP_P_T); 

 
#endif /* must be last line of file */



