// 
//  Copyright (C) 2000,2007  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef DS_DMDESCRIPTOR_H
#define DS_DMDESCRIPTOR_H

#ifndef DS_ALLDATATYPES_H
#include "dsAllDataTypes.hh"
#endif

extern "C" {

#ifndef DSLIB_H
#include "dslib.h"
#endif

#ifndef ASCDM_H
#include "ascdm.h"        /* AXAF Data Model lib */
#endif

}

extern dsDataTypes mtlConvertType(dmDataType type_2_convert_t);

extern dmDataType mtlConvertType(dsDataTypes type_2_convert_t);

extern void dsToLower(string &element);

extern void dsToUpper_char( char *element ); 

extern void dsToUpper(string &element); 

class dsDMDescriptor
{
public:

  /* constructors, destructors, assignment op */
  dsDMDescriptor(void);
  dsDMDescriptor(dmDescriptor *descr_p);
  ~dsDMDescriptor(void) {HelpClean();};
  dsDMDescriptor(const dsDMDescriptor &copy);
  dsDMDescriptor &operator = (const dsDMDescriptor &right);

  /* accessors */
  inline string GetUnits(void) {return units;};
  inline string GetDescription(void) {return description;};
  inline string GetName(void) {return name;};
  inline string GetLowerCaseName(void) {return lower_case_name;};
  inline dmDataType GetDMType(void) {return dm_data_type_e;};
  inline dsDataTypes GetDSType(void) {return ds_data_type_e;};
  inline dmDescriptorType GetDescrType(void) {return descr_type_e;};
  inline unsigned short IsVector(void) {return vector_flag;};
  inline dsDimensions GetDSDimensions(void) {return ds_dims;};
  inline long NumVectors(void) {return num_vectors;};
  inline unsigned short IsArray(void) {return array_flag;};
  inline long NumAxes(void) {return num_axes;};
  inline unsigned long GetDSArraySize(void) {return ds_dims.GetArraySize();};
  inline unsigned long GetDMArraySize(void) {return array_size;};
  inline long *GetAxes(void) {return axes_lengths_a;};
  inline unsigned char* GetBitData(void) {return (unsigned char*)data_holder_p;};
  inline char ** GetTextData(void) {return (char**)data_holder_p;};
  inline long GetCellLength(void) {return cell_length;};
  inline dmElementType GetDMElementType(void) {return element_type_e;};
  inline char ** GetVectorNames(void) {return vector_names_a;};
  void GetRange(dsAllDataTypes &min, dsAllDataTypes &max);

  inline string GetWCSName(void) { return wcs_name; };
  inline string GetWCSUnit(void) { return wcs_unit; };
  inline char **GetWCSColNames(void) { return cpt_names; };
  inline string GetWCSType(void) { return wcs_xform; };
  inline double *GetWCSPix(void) { return crpix; };
  inline double *GetWCSVal(void) { return crval;};
  inline double *GetWCSDelta(void) {return cdelt;};
  inline double *GetWCSPars(void) {return params;};
  inline long    GetWCSNumPars(void) {  return num_wcs_params;};

  /* modifiers */
  void SetUnits(string new_units) {units = new_units;};
  void SetDescription(string new_description){description = new_description;};
  void SetName(string new_name) {name = new_name; lower_case_name = name;
  				 dsToLower(lower_case_name);};
  void SetDescrType(dmDescriptorType new_descr_type_e) 
		{descr_type_e = new_descr_type_e;};
  void SetDMType(dmDataType new_dm_type_e);
  void SetNumVectors(long new_num_vectors, vector<string> vector_names);
  void SetAxes(long new_num_axes, long *new_axes_lengths_a);
  void SetAxes( dsDimensions new_axes);
  dsErrBool SetRange(dsAllDataTypes new_min, dsAllDataTypes new_max); 
  void SetCellLength(long new_cell_length); // this is important for bit and text
  void SetArraySize(long new_array_size); // this is important for bit and text

  void SetWCSName( string wname ) { wcs_name = wname; };
  void SetWCSUnit( string unit ) { wcs_unit = unit; };
  void SetWCSColNames( char **names );
  void SetWCSType( string type ) { wcs_xform = type; };
  void SetWCSPix( double *pix );
  void SetWCSVal( double *val );
  void SetWCSDelta( double *delta );
  void SetWCSPars( double *params );
  void SetWCSNumPars( long np ) { num_wcs_params = np; };
  inline unsigned short  IsRangeSet() { return range_set; }

 
protected:
  /* various helper functions */
  void HelpClean(void);
  void HelpCopy(const dsDMDescriptor &other);
  void AllocBitArray(void);
  void AllocTextArray(void);
  void AllocVectorNames(void);
  void DeleteBitArray(void);
  void DeleteTextArray(void);
  void DeleteVectorNames(void);
  void SetDSDimensions(void);

  /* data */
  string units;
  string description;
  string name;
  string lower_case_name;
  /* wcs */
  string wcs_name;
  string wcs_unit;
  char **cpt_names;
  string wcs_xform;
  double *crpix;
  double *crval;
  double *cdelt;
  double *params;
  long num_wcs_params;
  
  dmDescriptorType descr_type_e;
  dmDataType dm_data_type_e;
  dsDataTypes ds_data_type_e;
  long num_vectors;
  char **vector_names_a;
  dmElementType element_type_e;
  unsigned short vector_flag;
  unsigned short array_flag;
  long num_axes;
  long *axes_lengths_a;
  unsigned long array_size;
  dsDimensions ds_dims;
  unsigned short range_set;
  dsAllDataTypes range_min, range_max;

  /* these two will only be used for reading/writing 
     text and bit data */
  long cell_length;
  void *data_holder_p;

};

#endif
