#!/usr/bin/perl
# 
#  Copyright (C) 2004-2005,2007  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
unshift(@INC, $TEXEC_DIR);
require 'test_utils.pl';
$DM_TEST_EXCLUDE = find_inpath( "dm_test_exclude", ".:${TEXEC_DIR}", ':' );
$RM = find_inpath( "rm", $ENV{'PATH'}, ':' );
$MKDIR = find_inpath( "mkdir", $ENV{'PATH'}, ':' );
$PUNLEARN = find_inpath( 'punlearn', $ENV{'PATH'}, ':' );
$DMCOPY = find_inpath( 'dmcopy', $ENV{'PATH'}, ':' );

use strict;
use vars '$RM', '$MKDIR', '$PUNLEARN', '$ASCDS_INSTALL',
'$DM_TEST_EXCLUDE', '$DMCOPY';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '027';
my $semantic = 'exclude';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;
#
# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

# Process Date
my $date = `date +%d%b%Y_%H:%M:%S`;
chop($date);
my ($junk,$start_hours,$start_min,$start_sec ) = $date =~ /(.*_)(\d*):(\d*):(\d*)/;

# set platform
my $platform = `uname`;
chop $platform;


##########################################################################
# Initialize some default values
my $USER    = $ENV{'USER'};
my $UPARM   = $ENV{'ASCDS_USER_PARAM'};
my $version = $ENV{'ASCDS_VERSION'};
my $home    = $ENV{'HOME'};
my $thisdir = $ENV{'PWD'};
my $testin  = "$ENV{'TESTIN'}/datamodel";
my $testout = "$ENV{'TESTOUT'}/datamodel";
my $testlog = "$ENV{'TESTLOG'}/datamodel";

my $testsav;
if ( -d "$ENV{'TESTSAV'}/datamodel" )
{
   $testsav = "$ENV{'TESTSAV'}/datamodel";
}
else
{
   $testsav = "$ASCDS_INSTALL/test/baseline/datamodel";
}
my $testrun = $ENV{'TESTRUN'};
my $printfile = *STDOUT;
my $caldb   = $ENV{'CALDB'};

#
#------------------------------------------------------------
# Load Arguments
my %inargs;
%inargs = &parse_args(\@ARGV);

#
# Process Arguments

# ***********
# -h = Help
# ***********
if ( $inargs{"-h"} )
{
   print "Usage: \n";
   print "   $script_name [ -iosfph ] \n";
   print "      -h   Help ( this text ) \n";
   print "\n";
   print "      -o   Specify test output directory \n";
   print "\n";
   print "      -i   Specify test input directory \n";
   print "\n";
   print "      -s   Specify test saved output directory \n";
   print "\n";
   print "      -f   Specify log file \n";
   print "\n";
   print "      -p   Specify printed output file \n";
   print "\n";
   print "      -k   Specify kernel, e.g., FITS or TEXT \n";
   print "\n\n";
   exit 0;
}

# ***********
# -i = TESTIN directory
# ***********
if ( $inargs{'-i'} )
{
   $testin = $inargs{'-i'};
}
# ***********
# -o = TESTOUT directory
# ***********
if ( $inargs{'-o'} )
{
   $testout = $inargs{'-o'};
}
# ***********
# -s = TESTSAV directory
# ***********
if ( $inargs{'-s'} )
{
   $testsav = $inargs{'-s'};
}
# make sure testing directories exist
unless ( -d $testlog ) { system( "$MKDIR -p $testlog" ); }
unless ( -d $testout ) { system( "$MKDIR -p $testout" ); }
unless ( -d $testin )
{
   print $printfile "ERROR: can't find input directory : $testin\n";
   exit 1;
}
unless ( -d $testsav )
{
   print $printfile "ERROR: can't find saved output directory : $testsav\n";
   exit 1;
}

# Clean up old log files
my $comstr = sprintf( "$RM -f $testlog/dt%.3d.log.*", $id );
system( $comstr );

# Open log files

# ***********
# -p = verbose output file
# ***********
if ( $inargs{'-p'} )
{
   unless ( open PRINT_OUTPUT, ">>$inargs{'-p'}" )
   {
      print STDERR "ERROR: Failed to open verbose output file : $inargs{'-p'}\n";
      exit 1;
   }
   $printfile = *PRINT_OUTPUT;
   # turn off buffering to print file
   select($printfile); $| = 1;
}

my $logfile;
my $logfname = "$testlog/dt${id}.log.$date";
# ***********
# -f = LOG file
# ***********
if ( $inargs{'-f'} )
{
   $logfname = $inargs{'-f'};
}
if ( open LOG_OUTPUT, ">>$logfname" )
{
   $logfile = *LOG_OUTPUT;
}
else
{
   print $printfile "ERROR: Failed to open log file : $logfname\n";
   exit 1;
}
# turn off buffering for the logfile
select($logfile); $| = 1;
select($printfile);

# get specified test kernel
my $testKernel = $ARGV[0];

# ***********
# -k = Kernel
# ***********
# anything to do here?
if ( $inargs{'-k'} )
{
   $testKernel = $inargs{'-k'};
}

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
#my $tolfile = sprintf("$testin/tol%.3d", $id);
#my $tolkeysref = parse_tolfile( $tolfile );
#my @tolkeys = @$tolkeysref;
my @tolkeys;

######################### BEGIN TEST CODE ###################################

my $toolname = "dm_test_exclude"; # PROGRAMMER - put tool name here
my @testlist = 
( 
# PROGRAMMER - List the test ids here
 "1"
);
my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 "1" => { "text" => [ ] }
);
my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
# $version => "ASCDS_VERSION"
 
);

		     
#my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
my $toolpath = $DM_TEST_EXCLUDE;
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
#system( "$PUNLEARN $toolname" );


dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $time_user_total = 0;
my $time_sys_total = 0;
my $rc;
my $err_msg = "";

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf"); 
my $kernelName = "";
my $line = "";
my $pid;
my @kernels = "";
my $test_string;
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#########################################################################
# start the child process to get the kernel list
$pid = open( INHANDLE, "getkernellist |");

# get the output of the child process
$line = <INHANDLE>;

# break the output line into individual datum and put
# into list of kernel names
@kernels = split(/\s+/, $line);

close(INHANDLE);

# Control looping over each installed kernel
# by checking for kernel argument.
# If present, perform tests only the specified kernel
# If absent, perform tests on each installed kernel

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  if (($testKernel ne "") && ($kernelName ne $testKernel))
  {
    next;
  }

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    # Init per-test error flag
    my $mismatch = 0;
    $err_msg = "";
    my $time_user;
    my $time_sys;
    my $skipDelete = 0;

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";

    # if testing any kernel other than FITS, the FITS input files need to
    # be copied into the present kernel's format
    if ( $kernelName ne "FITS" )
    {
      my $sourceFile = "$testin/in${basename}.$source_ext";
      my $copyFile   = "$testin/in${basename}.$in_ext";
      my $options    = "opt=all";
      my $kernelOpts = $copyOpts{"$kernelName"};

      if ( ! -e "$infile" )
      {
        # copy fits file to text/dtf file
        # don't use opt=all - causes catastrophic error when copying
        # null primary block - same reason as above image copy problem

        my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$DMCOPY]\" $options";
        ($rc, $time_user, $time_sys) = system($teststring);

       $infile = $copyFile;
      }
      else
      {
        $skipDelete = 1;
      }
    }

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.txt";
    my $outfilefilt = "$testout/${basename}.txt_filt";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.txt";
    my $savfilefilt = "$testout/${basename}.txt_std";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";
    # PROGRAMMER - put test string here
    my $test_string = "$DM_TEST_EXCLUDE $infile $outfile";


    # run the tool
    $test_string .= " >> $logfname 2>$outfile";
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);

    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    #unless ( $rc == 0 )
    #{
    #   $err_msg = "ERROR: $toolname failed to run\n";
    #}
   
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "${savfile}_filt", \@tolkeys, $logfname, $test_diff, "" );
    }
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, $outfilefilt, $savfile, $savfilefilt, \%test_words, $logfname, $test_diff );
    }

    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }

    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );

    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ( $skipDelete == 0 )
      {
        system("$RM -f $infile");
      }        
    }
  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;

