#!/usr/bin/perl
# 
#  Copyright (C) 2008,2017  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#


#
# Add Test exec directory to INC path.
#
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
unshift(@INC, $TEXEC_DIR);
require 'test_utils.pl';

#
# Locate and set vars for executables used.
#
$RM       = find_inpath( "rm", $ENV{'PATH'}, ':' );
$MKDIR    = find_inpath( "mkdir", $ENV{'PATH'}, ':' );
$PUNLEARN = find_inpath( 'punlearn', $ENV{'PATH'}, ':' );
$DMCOPY   = find_inpath( 'dmcopy', $ENV{'PATH'}, ':' );
$DMLIST   = find_inpath( 'dmlist', $ENV{'PATH'}, ':' );

use strict;
use vars '$RM', '$MKDIR', '$PUNLEARN', '$ASCDS_INSTALL', '$DMCOPY', '$DMLIST';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '045';
my $semantic = 'asciiStrings';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;
#
# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

# Process Date
my $date = `date +%d%b%Y_%H:%M:%S`;
chop($date);
my ($junk,$start_hours,$start_min,$start_sec ) = $date =~ /(.*_)(\d*):(\d*):(\d*)/;

# set platform
my $platform = `uname`;
chop $platform;


##########################################################################
# Initialize some default values
my $USER    = $ENV{'USER'};
my $UPARM   = $ENV{'ASCDS_USER_PARAM'};
my $version = $ENV{'ASCDS_VERSION'};
my $home    = $ENV{'HOME'};
my $thisdir = $ENV{'PWD'};
my $testin  = "$ENV{'TESTIN'}/datamodel";
my $testout = "$ENV{'TESTOUT'}/datamodel";
my $testlog = "$ENV{'TESTLOG'}/datamodel";

my $testsav;
if ( -d "$ENV{'TESTSAV'}/datamodel" )
{
   $testsav = "$ENV{'TESTSAV'}/datamodel";
}
else
{
   $testsav = "$ASCDS_INSTALL/test/baseline/datamodel";
}
my $testrun = $ENV{'TESTRUN'};
my $printfile = *STDOUT;

#
#------------------------------------------------------------
# Load Arguments
my %inargs;
%inargs = &parse_args(\@ARGV);

#
# Process Arguments

# ***********
# -h = Help
# ***********
if ( $inargs{"-h"} )
{
   print "Usage: \n";
   print "   $script_name [ -iosfph ] \n";
   print "      -h   Help ( this text ) \n";
   print "\n";
   print "      -o   Specify test output directory \n";
   print "\n";
   print "      -i   Specify test input directory \n";
   print "\n";
   print "      -s   Specify test saved output directory \n";
   print "\n";
   print "      -f   Specify log file \n";
   print "\n";
   print "      -p   Specify printed output file \n";
   print "\n\n";
   exit 0;
}

# ***********
# -i = TESTIN directory
# ***********
if ( $inargs{'-i'} )
{
   $testin = $inargs{'-i'};
}
# ***********
# -o = TESTOUT directory
# ***********
if ( $inargs{'-o'} )
{
   $testout = $inargs{'-o'};
}
# ***********
# -s = TESTSAV directory
# ***********
if ( $inargs{'-s'} )
{
   $testsav = $inargs{'-s'};
}
# make sure testing directories exist
unless ( -d $testlog ) { system( "$MKDIR -p $testlog" ); }
unless ( -d $testout ) { system( "$MKDIR -p $testout" ); }
unless ( -d $testin )
{
   print $printfile "ERROR: can't find input directory : $testin\n";
   exit 1;
}
unless ( -d $testsav )
{
   print $printfile "ERROR: can't find saved output directory : $testsav\n";
   exit 1;
}

# Clean up old log files
my $comstr = sprintf( "$RM -f $testlog/dt%.3d.log.*", $id );
system( $comstr );

# Open log files

# ***********
# -p = verbose output file
# ***********
if ( $inargs{'-p'} )
{
   unless ( open PRINT_OUTPUT, ">>$inargs{'-p'}" )
   {
      print STDERR "ERROR: Failed to open verbose output file : $inargs{'-p'}\n";
      exit 1;
   }
   $printfile = *PRINT_OUTPUT;
   # turn off buffering to print file
   select($printfile); $| = 1;
}

my $logfile;
my $logfname = "$testlog/dt${id}.log.$date";
# ***********
# -f = LOG file
# ***********
if ( $inargs{'-f'} )
{
   $logfname = $inargs{'-f'};
}
if ( open LOG_OUTPUT, ">>$logfname" )
{
   $logfile = *LOG_OUTPUT;
}
else
{
   print $printfile "ERROR: Failed to open log file : $logfname\n";
   exit 1;
}
# turn off buffering for the logfile
select($logfile); $| = 1;
select($printfile);

#
# Parse the tolerance file
#   (This is only necessary if this test uses the dmlist_test.)
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################
my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = 
( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11,
12,
13,
14,
15,
16,
);

my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 1 => { "text" => [] },
 2 => { "text" => [] },
 3 => { "text" => [] },
 4 => { "text" => [] },
 5 => { "text" => [] },
 6 => { "text" => [] },
 7 => { "text" => [] },
 8 => { "text" => [] },
 9 => { "text" => [] },
10 => { "text" => [] },
11 => { "text" => [] },
12 => { "text" => [] },
13 => { "text" => [] },
14 => { "text" => [] },
15 => { "text" => [] },
16 => { "text" => [] },
);

my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# eg:
$version => "ASCDS_VERSION",
'\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d' => "TEST_DATE",
'HISTORY.*\n' => "",
'HISTNUM.*' => "HISTNUM",
 'NaN' => "NAN",
 'nan\s' => "NAN ",
$testin  => "<TESTIN>"
);

my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );


dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my $ii = 0;
my @faillist = ( );
my $time_user_total = 0;
my $time_sys_total = 0;
my $rc;
my $err_msg = "";

#########################################################################
# Loop over all tests
foreach my $testid (@testlist)
{
   # Init per-test error flag
   my $mismatch = 0;
   $err_msg = "";
   my $time_user;
   my $time_sys;

   # delete old outputs
   my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
   system("$RM -f $testout/${basename}*");

   my $infile = "$testin/in${basename}.txt";
   # PROGRAMMER - put output filename here
   my $outfile = "$testout/${basename}.txt";
   # PROGRAMMER - put output filter filename here
   my $outfile_filt = "$testout/${basename}.txt_filt";
   # PROGRAMMER - put saved output filename here
   my $savfile = "$testsav/${basename}.txt";
   # PROGRAMMER - put name of diff file here, if needed
   my $test_diff = "$testout/${basename}.diff";
   # PROGRAMMER - put saved output filter filename here
   my $savfile_filt = "$testout/${basename}.txt_savfilt";
   # PROGRAMMER - put test string here
   my $test_string = "";

   # Test Descriptions:
   #  These tests work the string handling logic of the ascii kernel.
   #  T1/T2:
   #    DTF-FIXED format input with TBCOL/TDISP keys defining fields.
   #    values are single/multi-word, quoted/unquoted, with and without
   #    significant whitespace.
   #
   #  T3/T4:
   #    DTF format input with TFORM keys defining fields and ',' delimited.
   #    values are single/multi-word, quoted/unquoted, with and without
   #    significant whitespace.
   #
   #  T5:
   #    Tests how the DM interprets string column data.. each instance has
   #    various significant/insignificant quotes to handle.
   #
   if ( $testid == 1 )
   {
      # DTF-FIXED -> DTF;  sep == ','; with TDISP keys
      $test_string = "$DMCOPY infile=$infile outfile=$outfile'[opt kernel=text/dtf,sep=\",\"]'";
   }
   elsif ( $testid == 2 )
   {
      # DTF-FIXED -> DTF;  sep == ' '; with TDISP keys
      $test_string = "$DMCOPY infile=$infile outfile=$outfile'[opt kernel=text/dtf]'";
   }
   elsif ( $testid == 3 )
   {
      # DTF -> DTF;  sep == ','
      $test_string = "$DMCOPY infile=$infile'[opt sep=\",\"]' outfile=$outfile'[opt kernel=text/dtf,sep=\",\"]'";
   }
   elsif ( $testid == 4 )
   {
      # DTF-FIXED -> DTF;  sep == ' '
      $test_string = "$DMCOPY infile=$infile'[opt sep=\",\"]' outfile=$outfile'[opt kernel=text/dtf]'";
   }
   elsif ( $testid == 5 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=0]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 6 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=1]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 7 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=2]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 8 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=3]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 9 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=4]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 10 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=5]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 11 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=6]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 12 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=7]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 13 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=8]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 14 )
   {
      # interp of string columns, various quote scenarios
      $test_string = "$DMLIST infile=$infile'[opt skip=9]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 15 )
   {
      # interp of string columns, escaped separators.
      $test_string = "$DMLIST infile=$infile'[opt skip=10]' outfile=$outfile rows=1:1 opt=cols,data";
   }
   elsif ( $testid == 16 )
   {
      # SL-99 date to string + check consolidation preserves previous behavior
      $test_string = "$DMLIST infile=$infile'[opt colnames=last]' outfile=$outfile opt=cols";
   }

   $test_string .= " >> $logfname 2>&1";

   dt_print_test_header( $testid, $test_string, $logfile, $printfile );
   close $logfile;

   # run the tool
   ($rc, $time_user, $time_sys) = dt_run_test($test_string);

   open $logfile, ">>$logfname";
   select($logfile); $| = 1; select($printfile);
   unless ( $rc == 0 )
   {
      $err_msg = "ERROR: $toolname failed to run\n";
   }

   my $block_typesref = $test_blocks{$testid};
   my %block_types = %$block_typesref;
   if ( $block_types{"table"} )
   {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"image"} )
   {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"list"} )
   {
      $err_msg .= dmlist_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \@tolkeys, $logfname, $test_diff, "" );
   }
   if ( $block_types{"text"} )
   {
      $err_msg .= diff_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \%test_words, $logfname, $test_diff );
   }

   unless ( $err_msg eq "" )
   {
      push( @faillist, $testid );
      $mismatch = 1;
   }
   $time_user_total += $time_user;
   $time_sys_total  += $time_sys;
   dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );

   $ii++;
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
