#!/usr/bin/perl
# 
#  Copyright (C) 2014,2017  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
unshift(@INC, $TEXEC_DIR);
require 'test_utils.pl';
$RM = find_inpath( "rm", $ENV{'PATH'}, ':' );
$MKDIR = find_inpath( "mkdir", $ENV{'PATH'}, ':' );
$PUNLEARN = find_inpath( 'punlearn', $ENV{'PATH'}, ':' );
$DMCOPY = find_inpath( 'dmcopy', $ENV{'PATH'}, ':' );
$DMLIST = find_inpath( 'dmlist', $ENV{'PATH'}, ':' );

use strict;
use vars '$RM', '$MKDIR', '$PUNLEARN', '$ASCDS_INSTALL', '$DMCOPY', '$DMLIST';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '047';
my $semantic = 'boolean3stat';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;
#
# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";
$ENV{'LC_ALL'} = "C";

# Process Date
my $date = `date +%d%b%Y_%H:%M:%S`;
chop($date);
my ($junk,$start_hours,$start_min,$start_sec ) = $date =~ /(.*_)(\d*):(\d*):(\d*)/;

# set platform
my $platform = `uname`;
chop $platform;


##########################################################################
# Initialize some default values
my $USER    = $ENV{'USER'};
my $UPARM   = $ENV{'ASCDS_USER_PARAM'};
my $version = $ENV{'ASCDS_VERSION'};
my $home    = $ENV{'HOME'};
my $thisdir = $ENV{'PWD'};
my $testin  = "$ENV{'TESTIN'}/datamodel";
my $testout = "$ENV{'TESTOUT'}/datamodel";
my $testlog = "$ENV{'TESTLOG'}/datamodel";

my $testsav;
if ( -d "$ENV{'TESTSAV'}/datamodel" )
{
   $testsav = "$ENV{'TESTSAV'}/datamodel";
}
else
{
   $testsav = "$ASCDS_INSTALL/test/baseline/datamodel";
}
my $testrun = $ENV{'TESTRUN'};
my $printfile = *STDOUT;
my $caldb   = $ENV{'CALDB'};

#
#------------------------------------------------------------
# Load Arguments
my %inargs;
%inargs = &parse_args(\@ARGV);

#
# Process Arguments

# ***********
# -h = Help
# ***********
if ( $inargs{"-h"} )
{
   print "Usage: \n";
   print "   $script_name [ -iosfph ] \n";
   print "      -h   Help ( this text ) \n";
   print "\n";
   print "      -o   Specify test output directory \n";
   print "\n";
   print "      -i   Specify test input directory \n";
   print "\n";
   print "      -s   Specify test saved output directory \n";
   print "\n";
   print "      -f   Specify log file \n";
   print "\n";
   print "      -p   Specify printed output file \n";
   print "\n";
   print "      -k   Specify kernel, e.g., FITS or TEXT \n";
   print "\n\n";
   exit 0;
}

# ***********
# -i = TESTIN directory
# ***********
if ( $inargs{'-i'} )
{
   $testin = $inargs{'-i'};
}
# ***********
# -o = TESTOUT directory
# ***********
if ( $inargs{'-o'} )
{
   $testout = $inargs{'-o'};
}
# ***********
# -s = TESTSAV directory
# ***********
if ( $inargs{'-s'} )
{
   $testsav = $inargs{'-s'};
}
# make sure testing directories exist
unless ( -d $testlog ) { system( "$MKDIR -p $testlog" ); }
unless ( -d $testout ) { system( "$MKDIR -p $testout" ); }
unless ( -d $testin )
{
   print $printfile "ERROR: can't find input directory : $testin\n";
   exit 1;
}
unless ( -d $testsav )
{
   print $printfile "ERROR: can't find saved output directory : $testsav\n";
   exit 1;
}

# Clean up old log files
my $comstr = sprintf( "$RM -f $testlog/dt%.3d.log.*", $id );
system( $comstr );

# Open log files

# ***********
# -p = verbose output file
# ***********
if ( $inargs{'-p'} )
{
   unless ( open PRINT_OUTPUT, ">>$inargs{'-p'}" )
   {
      print STDERR "ERROR: Failed to open verbose output file : $inargs{'-p'}\n";
      exit 1;
   }
   $printfile = *PRINT_OUTPUT;
   # turn off buffering to print file
   select($printfile); $| = 1;
}

my $logfile;
my $logfname = "$testlog/dt${id}.log.$date";
# ***********
# -f = LOG file
# ***********
if ( $inargs{'-f'} )
{
   $logfname = $inargs{'-f'};
}
if ( open LOG_OUTPUT, ">>$logfname" )
{
   $logfile = *LOG_OUTPUT;
}
else
{
   print $printfile "ERROR: Failed to open log file : $logfname\n";
   exit 1;
}
# turn off buffering for the logfile
select($logfile); $| = 1;
select($printfile);

# get specified test kernel
my $testKernel = $ARGV[0];

# ***********
# -k = Kernel
# ***********
# anything to do here?
if ( $inargs{'-k'} )
{
   #$testKernel = $inargs{'-k'};
}

$testKernel = "FITS";

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################

my $toolname = ""; # PROGRAMMER - put tool name here
my @testlist = 
( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
21,
22,
23,
31,
32,
33,
41,
42,
43,
44,
45,
51,
52,
53,
54,
55,
56,
57,
65,
66,
67,
68,
69,
71,
72,
73,
);
my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],    <<= dmimgcalc test
#          "table" => [ 2, 4 ] },  <<= dmdiff test
# "2" => { "list" => [ ] },...
1 => { "table" => [1,2] },
2 => { "table" => [1,2] },
3 => { "table" => [1,2] },
4 => { "table" => [1,2] },
5 => { "table" => [1,2] },
6 => { "table" => [1,2] },
7 => { "table" => [1,2] },
8 => { "table" => [1,2] },
9 => { "table" => [1,2] },
10 => { "table" => [1,2] },
21 => { "table" => [1,2] },
22 => { "table" => [1,2] },
23 => { "table" => [1,2] },
31 => { "table" => [1,2]},
32 => { "table" => [1,2]},
33 => { "table" => [1,2]},
41 => { "text2" => [] },
42 => { "text" => [] },
43 => { "text" => [] },
44 => { "text" => [] },
45 => { "text" => [] },
51 => { "fail" => []},       # expected failure.
52 => { "fail" => []},       # expected failure.
53 => { "fail" => []},       # expected failure.
54 => { "fail" => []},       # expected failure.
55 => { "fail" => []},       # expected failure.
56 => { "fail" => []},       # expected failure.
57 => { "fail" => []},       # expected failure.
65 => { "text" => [] },
66 => { "text2" => [] },
67 => { "text" => [] },
68 => { "table" => [2] },
69 => { "text2"  => [] },
71 => { "table" => [2]},
72 => { "table" => [2]},
73 => { "table" => [2]},
);

my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
# ** DTF HISTORY format **
$version => "ASCDS_VERSION",
'HISTORY.*\n' => "",
'HISTNUM.*\n' => ""
);
my %test_tsv_words = 
(
# ** TSV HISTORY format **
$version => "ASCDS_VERSION",
'#HISTORY.*\n' => "",
'HISTNUM.*\n' => ""
);


# Set parameters to defaults
system( "$PUNLEARN dmcopy" );
system( "$PUNLEARN dmlist" );


dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $time_user_total = 0;
my $time_sys_total = 0;
my $rc;
my $err_msg = "";

my %exten = ("FITS" => "fits");
my $kernelName = "";
my $line = "";
my $pid;
my @kernels = "";
my $test_string;
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#######################################################################
# start the child process to get the kernel list
$pid = open( INHANDLE, "getkernellist |");

# get the output of the child process
$line = <INHANDLE>;

# break the output line into individual datum and put
# into list of kernel names
@kernels = split(/\s+/, $line);

close(INHANDLE);

# Control looping over each installed kernel
# by checking for kernel argument.
# If present, perform tests only the specified kernel
# If absent, perform tests on each installed kernel

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  if (($testKernel ne "") && ($kernelName ne $testKernel))
  {
    next;
  }

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    if ( $test_blocks{$testid} == "" ){ next; }

    # Init per-test error flag
    my $mismatch = 0;
    $err_msg = "";
    my $time_user;
    my $time_sys;
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;

    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.fits";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.fits";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";
    # PROGRAMMER - put test string here
    my $test_string = "";


    # add test commands here

    # -------------------------------------------------------
    # Basic tests: 3-state boolean values
    # -------------------------------------------------------
    if ( $testid == 1 )
    {
	$test_string = "$DMCOPY infile=\"${infile}[double=T]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 2 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=y]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 1;
	$savfile = "$testsav/${basename_ref}.fits";
   }
    if ( $testid == 3 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=1]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 1;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    if ( $testid == 4 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=f]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 5 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=N]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 4;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    if ( $testid == 6 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=0]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 4;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    if ( $testid == 7 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=U]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 8 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=u]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 7;
	$savfile = "$testsav/${basename_ref}.fits";

    }
    if ( $testid == 9 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=2]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 7;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    if ( $testid == 10 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=null]\" outfile=$outfile verbose=0";
	my $basename_ref = sprintf "dt%.3dm%.3d", $id, 7;
	$savfile = "$testsav/${basename_ref}.fits";
    }
    # -------------------------------------------------------
    # Basic tests: list filter
    # test-21: copy rows containing TRUE and NULL value
    # test-22: copy rows containing FALSE and NULL value
    # test-23: copy rows containing FALSE, TRUE and NULL value
    # -------------------------------------------------------
    if ( $testid == 21 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=1,2]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 22 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=n,u]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 23 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 1;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string = "$DMCOPY infile=\"${infile}[double=0,Y,U]\" outfile=$outfile verbose=0";
    }

    # -------------------------------------------------------
    # Basic tests: with filter in row range
    # test-31: infile with nulll value as B
    # test-32: infile with nulll value as N
    # test-33: infile with nulll value as U
    # -------------------------------------------------------
    if ( $testid == 31 )
    {
 	#-- Note:  infile with null value as 'B' 
	$test_string = "$DMCOPY infile=\"${infile}[#row=1:106]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 32 )
    {
 	#-- Note:  infile with null value as 'N'
	$test_string = "$DMCOPY infile=\"${infile}[#row=1:106]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 33 )
    {
	#-- Note:  infile with null value as 'U'
 	$test_string = "$DMCOPY infile=\"${infile}[#row=1:106]\" outfile=$outfile verbose=0";
    }

    # -------------------------------------------------------
    # Basic tests: text/raw and text/simple format with NULLSTR option
    # test-41: infile with nulll value as B
    # test-42: infile with nulll value as N
    # test-43: infile with nulll value as U
    # -------------------------------------------------------
     if ( $testid == 41 )
    {	
	$infile  = "$testin/in${basename}.dtf";
	$outfile = "$testout/${basename}.tsv";
	$savfile = "$testsav/${basename}.tsv";

 	# test read/write dtf/tsv logical column for infile with NULLSTR=- and output file with NULLSTR=NULL
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=-]\" outfile=$outfile\"[opt kernel=text/tsv,NULLSTR=NULL]\" verbose=0";
    }
    if ( $testid == 42 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 41;
	$infile  = "$testin/in${basename_in}.dtf";
	$outfile = "$testout/${basename}.raw";
	$savfile = "$testsav/${basename}.raw";

 	# test read/write dtf/raw logical column for infile with NULLSTR=- and output file with NULLSTR=NULL
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=-]\" outfile=$outfile\"[opt kernel=text/raw,NULLSTR=NULL]\" verbose=0";
    }
    if ( $testid == 43 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 41;
	$infile  = "$testin/in${basename_in}.dtf";
	$outfile = "$testout/${basename}.dtf";
	$savfile = "$testsav/${basename}.dtf";

 	# test read/write dtf/dtf logical column for infile with NULLSTR=- and output file with NULLSTR=NULL
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=-]\" outfile=$outfile\"[opt kernel=text/dtf,NULLSTR=NULL]\" verbose=0";
    }
    if ( $testid == 44 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 41;
	$infile  = "$testin/in${basename_in}.dtf";
	$outfile = "$testout/${basename}.txt";
	$savfile = "$testsav/${basename}.txt";

 	# test read/write dtf/txt logical column for infile with NULLSTR=- and output file with NULLSTR=NULL
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=-]\" outfile=$outfile\"[opt kernel=text/sm,NULLSTR=NULL]\" verbose=0";
    }
    if ( $testid == 45 )
    {
	$infile  = "$testin/in${basename}.raw";
	$outfile = "$testout/${basename}.txt";
	$savfile = "$testsav/${basename}.txt";

 	# test read/write raw/txt logical column for infile with NULLSTR=NULL and output file with NULLSTR=H
	$test_string = "$DMCOPY infile=\"${infile}[opt NULLSTR=NULL]\" outfile=$outfile\"[opt kernel=text/simple,NULLSTR=H]\" verbose=0";
    }

   # -------------------------------------------------------
    # Basic tests: expected bad returns
    # -------------------------------------------------------
    if ( $testid == 51 )
    {
	$test_string  = "$DMLIST infile=\"${infile}[double=0:4]\" opt=counts outfile=$outfile verbose=0";
    }
    if ( $testid == 52 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double=0:1]\" opt=counts outfile=$outfile verbose=0";
    }
    if ( $testid == 53 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double=2:]\" opt=counts outfile=$outfile verbose=0";
    }

    if ( $testid == 54 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double >= Y]\" opt=counts outfile=$outfile  verbose=0";
    }

    if ( $testid == 55 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double <= U]\" opt=counts outfile=$outfile  verbose=0";
    }

    if ( $testid == 56 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double > n]\" opt=counts outfile=$outfile verbose=0";
    }
 
    if ( $testid == 57 )
    {
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.fits";
	$test_string  = "$DMLIST infile=\"${infile}[double < 2]\" opt=counts outfile=$outfile  verbose=0";
    }
    if ( $testid == 65 )
    {   
	$infile  = "$testin/in${basename}.tsv\"[opt kernel=text/tsv]\"";
	$outfile = "$testout/${basename}.txt";
	$savfile = "$testsav/${basename}.txt";
	$test_string = "$DMCOPY infile=${infile} outfile=$outfile\"[opt kernel=text/simple]\" verbose=0 ";
    }
    if ( $testid == 66 )
    {   
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 65;
	$infile  = "$testin/in${basename_in}.tsv\"[opt kernel=text/tsv]\"";
	$outfile = "$testout/${basename}.tsv";
	$savfile = "$testsav/${basename}.tsv";
	$test_string = "$DMCOPY infile=${infile} outfile=$outfile\"[opt kernel=text/tsv]\" verbose=0 ";
    }
    if ( $testid == 67 )
    {   
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 65;
	$infile  = "$testin/in${basename_in}.tsv\"[opt kernel=text/tsv]\"";
	$outfile = "$testout/${basename}.dtf";
	$savfile = "$testsav/${basename}.dtf";
	$test_string  = "$DMCOPY infile=${infile} outfile=$outfile\"[opt kernel=text/dtf]\" verbose=0 ";
    }

    if ( $testid == 68 )
    {   
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 65;
	$infile  = "$testin/in${basename_in}.tsv\"[opt kernel=text/tsv]\"";
	$test_string  = "$DMCOPY infile=${infile} outfile=$outfile verbose=0 ";
    }

    if ( $testid == 69 )
    {   
	my $basename_in = sprintf "dt%.3dm%.3d", $id, 51;
	$infile  = "$testin/in${basename_in}.$in_ext";
	$outfile = "$testout/${basename}.tsv";
	$savfile = "$testsav/${basename}.tsv";
	$test_string  = "$DMCOPY infile=${infile} outfile=$outfile\"[opt kernel=text/tsv]\" verbose=0 ";
    }
 
   if ( $testid == 71 )
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 1;
	$infile   = "$testin/in${basename}.$in_ext";
	
	$test_string = "$DMCOPY infile=\"${infile}[exclude double=t]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 72 )
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 1;
	$infile   = "$testin/in${basename}.$in_ext";
	$test_string = "$DMCOPY infile=\"${infile}[exclude double=0]\" outfile=$outfile verbose=0";
    }
    if ( $testid == 73 )
    {
	$basename = sprintf "dt%.3dm%.3d", $id, 1;
	$infile   = "$testin/in${basename}.$in_ext";
	$test_string = "$DMCOPY infile=\"${infile}[exclude double=U]\" outfile=$outfile verbose=0";
    }
 
    # --------------------------------------------------------------------------------
    #
    # run the tool
    $test_string .= " >> $logfname 2>&1";
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    ($rc, $time_user, $time_sys) = dt_run_test($test_string);

    if ( $block_types{"fail"} )
    { # Expected Failure test check.
        if ( $rc == 0 )
	{
	    $err_msg = "  ERROR: Expected failure did not occur.\n";
	}
	$rc = 0;
    }

    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 )
    {
      $err_msg = "ERROR: $toolname failed to run\n";
    }
   
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff ); 
     }
    }
    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
    }
    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt_list", $savfile, "$testout/${basename}_savfilt_list", \@tolkeys, $logfname, $test_diff, "" );
    }
    if ( $block_types{"text"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_filt_text", $savfile, "$testout/${basename}_savfilt_text", \%test_words, $logfname, $test_diff );
    }
    if ( $block_types{"text2"} )
    {
      $err_msg .= diff_test( $outfile, "${outfile}_filt_text", $savfile, "$testout/${basename}_savfilt_text", \%test_tsv_words, $logfname, $test_diff );
    }

    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        #push (@faillist, $kernelName);
        #$errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }

    $time_user_total += $time_user;
    $time_sys_total  += $time_sys;
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );

    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ( $skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
    }

  }
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
