#!/usr/bin/env python
from __future__ import print_function
from pycrates import * 
import numpy as np
import sys
import os


class PycratesSmoketests:

    def __init__(self, *args):

        if 'help' in args or '-h' in args:
            print(self.usage())
            return

        self.tests_to_run = []
        self.detail = False

        if 'OUTDIR' in os.environ:
            self.outdir = os.environ['OUTDIR']
        else:
            self.outdir = "./"

        all_tests = ['table', 'image', 'ds', 'pha1', 'pha2', 'rmf']

        if '-v' in args:
            self.detail = True

        if len(args) == 0 or ('-v' in args and len(args) == 1):
            self.tests_to_run = all_tests
            return

        for item in args:
            item_lc = item.lower()
  
            if item_lc != '-v' and item_lc != '-h' and item_lc != 'help':

                if item_lc == 'all':
                    self.tests_to_run = all_tests
                    break

                elif item_lc in all_tests:
                    self.tests_to_run.append( item_lc )
                
                else:  # for invalid args 
                    print(self.usage())
                    break


    def usage(self):
        retstr = "Usage: pycrates-smoketest.py [-v] [all] [ds] [image] [pha1] [pha2] [rmf] [table]"
        return retstr


    def run_tests(self):

        status = "PASS"

        for testname in self.tests_to_run:
            teststat = eval( "self.run_"+testname+"()" )
            if teststat == "FAIL":
                status = "FAIL"
            if self.detail:
                print(teststat)
         
        return status


    def run_ds(self):
        filename = self.outdir + "/Dataset.fits"
        ds = CrateDataset()
        status = ds._self_test(filename, debug=self.detail)
        return status

    def run_image(self):
        filename = self.outdir + "/Image.fits"
        img = IMAGECrate()
        status = img._self_test(filename, debug=self.detail)
        return status

    def run_pha1(self):
        filename = self.outdir + "/PHA1.fits"
        pha = PHACrateDataset()
        status = pha._self_test(filename, debug=self.detail, pha1=True)
        return status

    def run_pha2(self):

        filename = self.outdir + "/PHA2.fits"
        pha = PHACrateDataset()
        status = pha._self_test(filename, debug=self.detail, pha1=False)
        return status

    def run_rmf(self):
        filename = self.outdir + "/RMF.fits"
        rmf = RMFCrateDataset()
        status = rmf._self_test(filename, debug=self.detail)
        return status

    def run_table(self):
        filename = self.outdir + "/Table.fits"
        tab = TABLECrate()
        status = tab._self_test(filename, debug=self.detail)
        return status



if __name__ == '__main__':

    t = PycratesSmoketests(*sys.argv[1:])
    
    try:
        status = t.run_tests()

        pathdir = os.environ['ASCDS_WORK_PATH']
        logname = os.environ['LOGNAME']
        testid  = os.environ['TESTID']
        grade_file = pathdir+"/smoke." + logname + "/" + testid + "/grade.txt"

        grade_fh = open(grade_file, 'w')
        print(status, file=grade_fh)
        grade_fh.close()

    except Exception as ex:
        pass
